// Copyright© 2019 HP Development Company, L.P.  
// Subject to the Terms of Use on the HP Developer's Portal at https://developers.hp.com/terms
(function() {
  'use strict';

  // This config stores the important strings needed to
  // connect to the HP Techpulse API using OAuth service
  var config = {
      clientId: 'CLIENT_ID',
      redirectUri: 'http://localhost:8087/redirect',
      authUrl: 'https://daas.api.hp.com',
      version: '20190102'
  };

  // Called when web page first loads and when
  // the OAuth flow returns to the page
  //
  // This function parses the access token in the Cookies if available
  // It also adds a link to the HP DaaS Login button
  $(document).ready(function() {
      var accessToken = Cookies.get("accessToken");
      var hasAuth = accessToken && accessToken.length > 0 && accessToken != "undefined";
      updateUIWithAuthState(hasAuth);

      $("#connectbutton").click(function() {
          doAuthRedirect();
      });

      $("#getdatabutton").click(function() {
          tableau.connectionName = "HP TechPulse Analytics Hardware Inventory Data";
          tableau.submit();
      });
  });

  // An on-click function for HP DaaS Login button,
  // This will redirect the user to a HP DaaS login
  function doAuthRedirect() {
      var appId = config.clientId;
      if (tableau.authPurpose === tableau.authPurposeEnum.ephemerel) {
        appId = config.clientId;
      } else if (tableau.authPurpose === tableau.authPurposeEnum.enduring) {
        appId = config.clientId; // This should be the Tableau Server appID
      }

	  var url = config.authUrl + '/oauth/v1/authorize?response_type=code&client_id=' + appId + '&redirect_uri=' + config.redirectUri + '&scope=Read&state=daas_session';
      window.location.href = url;
  }

  //------------- OAuth Helpers -------------//
  // This helper function returns the URI for Techpulse Analytics Hardware Inventory endpoint
  // It appends the passed in accessToken to the call to personalize the call for the user
  function getTechpulseApi(accessToken) {
	  return "https://daas.api.hp.com/analytics/v1/reports/hwinv/details/type/grid";
  }

  // This function toggles the label shown depending
  // on whether or not the user has been authenticated
  function updateUIWithAuthState(hasAuth) {
      if (hasAuth) {
          $(".notsignedin").css('display', 'none');
          $(".signedin").css('display', 'block');
      } else {
          $(".notsignedin").css('display', 'block');
          $(".signedin").css('display', 'none');
      }
  }

  //------------- Tableau WDC code -------------//
  // Create tableau connector, should be called first
  var myConnector = tableau.makeConnector();

  // Init function for connector, called during every phase but
  // only called when running inside the simulator or tableau
  myConnector.init = function(initCallback) {
      tableau.authType = tableau.authTypeEnum.custom;

      // If we are in the auth phase we only want to show the UI needed for auth
      if (tableau.phase == tableau.phaseEnum.authPhase) {
        $("#getdatabutton").css('display', 'none');
      }

      if (tableau.phase == tableau.phaseEnum.gatherDataPhase) {
        // If API that WDC is using has an endpoint that checks
        // the validity of an access token, that could be used here.
        // Then the WDC can call tableau.abortForAuth if that access token
        // is invalid.
      }

      var accessToken = Cookies.get("accessToken");
      console.log("Access token is '" + accessToken + "'");
      var hasAuth = (accessToken && accessToken.length > 0) || tableau.password.length > 0;
      updateUIWithAuthState(hasAuth);

      initCallback();

      // If we are not in the data gathering phase, we want to store the token
      // This allows us to access the token in the data gathering phase
      if (tableau.phase == tableau.phaseEnum.interactivePhase || tableau.phase == tableau.phaseEnum.authPhase) {
          if (hasAuth) {
              tableau.password = accessToken;

              if (tableau.phase == tableau.phaseEnum.authPhase) {
                // Auto-submit here if we are in the auth phase
                tableau.submit()
              }

              return;
          }
      }
  };

  // Declare the data to Tableau that we are returning from 
  // HP TechPulse API for Harware Inventory Details
  myConnector.getSchema = function(schemaCallback) {
      var schema = [];

      var col1 = { id: "DeviceName", dataType: "string"};
      var col2 = { id: "DeviceType", dataType: "string"};
      var col3 = { id: "DeviceManufacturer", dataType: "string"};
      var col4 = { id: "DeviceModel", dataType: "string"};
      var col5 = { id: "SerialNumber", dataType: "string"};
      var col6 = { id: "OperatingSystem", dataType: "string"};      
      var col7 = { id: "CompanyName", dataType: "string"};
      var col8 = { id: "CounterId", dataType: "string"};
      var col9 = { id: "DataAsOfDate", dataType: "string"};
      var col10 = { id: "EnrolledDate", dataType: "string"};
      var col11 = { id: "Month", dataType: "string"};
      var col12 = { id: "DeviceId", dataType: "string"};      
      var col13 = { id: "OriginalOS", dataType: "string"};
      var col14 = { id: "Graphics", dataType: "string"};
      var col15 = { id: "LastOnlineDate", dataType: "string"};
      var col16 = { id: "Country", dataType: "string"};
      var col17 = { id: "Memory", dataType: "string"};
      var col18 = { id: "ManufactureDate", dataType: "string"};
      var col19 = { id: "OSStatus", dataType: "string"};
      var col20 = { id: "OSRelease", dataType: "string"};
      var col21 = { id: "OSReleselToggaleEnabled", dataType: "string"};
      var col22 = { id: "OSVersion", dataType: "string"};
      var col23 = { id: "OSVersionStatusToggleEnabled", dataType: "string"};
      var col24 = { id: "Processor", dataType: "string"};
	  
	  
      var cols = [col1, col2, col3, col4, col5, col6, col7, col8, col9, col10, col11, col12, col13, col14, col15, col16, col17, col18, col19, col20, col21, col22, col23, col24];

      var tableInfo = {
        id: "HP_TechPulse_Analytics_Table",
        columns: cols
      }

      schema.push(tableInfo);
      schemaCallback(schema);
  };

  // This function actually makes the HP TechPulse Hardware Inventory Details API call 
  // parses the results and passes them back to Tableau
  myConnector.getData = function(table, doneCallback) {
      var dataToReturn = [];
      var hasMoreData = false;

      var accessToken = tableau.password;
      var connectionUri = getTechpulseApi(accessToken);

      var xhr = $.ajax({
		  method: 'POST',
          url: connectionUri,
          dataType: 'json',
		  headers: {
				"Authorization": "Bearer " + accessToken
          },
          success: function (data) {
              if (data.resources) {
                  var resources = data.resources;

					var ii;
					for (ii = 0; ii < resources.length; ++ii) {
						
 						var venue = {
								'DeviceName' : resources[ii].deviceName,
								'DeviceType' : resources[ii].deviceType,
								'DeviceManufacturer' : resources[ii].deviceMfg,
								'DeviceModel' : resources[ii].deviceModel,
								'SerialNumber' : resources[ii].deviceSn,
								'OperatingSystem' : resources[ii].deviceOs,
								'CompanyName' : resources[ii].companyName,
								'CounterId' : resources[ii].counterId,
								'DataAsOfDate' : resources[ii].dataAsOfDate,
								'EnrolledDate' : resources[ii].dateEnrolled,
								'Month' : resources[ii].dateListMonth,
								'DeviceId' : resources[ii].deviceId,
								'OriginalOS' : resources[ii].deviceOsOriginal,
								'Graphics' : resources[ii].graphics,
								'LastOnlineDate' : resources[ii].lastOnlineDate,
								'Country' : resources[ii].locationcc,
								'Memory' : resources[ii].memory,
								'ManufactureDate' : resources[ii].mfgDate,
								'OSStatus' : resources[ii].osStatus,
								'OSRelease' : resources[ii].osrelease,
								'OSReleselToggaleEnabled' : resources[ii].osreleselToggaleEnabled,
								'OSVersion' : resources[ii].osversion,
								'OSVersionStatusToggleEnabled' : resources[ii].osversionStatusToggleEnabled,
								'Processor' : resources[ii].processor
							};
							dataToReturn.push(venue);
					}

					table.appendRows(dataToReturn);
					doneCallback();
              }
              else {
                  tableau.abortWithError("No results found");
              }
          },
          error: function (xhr, ajaxOptions, thrownError) {
              // WDC should do more granular error checking here
              // or on the server side.  This is just a sample of new API.
              tableau.abortForAuth("Invalid Access Token");
          }
      });
  };

  // Register the tableau connector, call this last
  tableau.registerConnector(myConnector);
})();
